SUBSCHEMA GeoGeant  AUTHOR 'M.Dueren'

/*

  Most sizes are half size (half lengths and radii)

*/

DEFINE ESET
dgMATE=(
  cName     = CH16                     : 'material name',
  rA        = REAL [0.,300.0]          : 'atomic weight',
  rZ        = REAL [0.,300.0]          : 'atomic number',
  rDens     = REAL [0.,30.000]           : 'density',
  rRadl     = REAL [0.000,*]             : 'rad. length')

 : 'basic material definitions';
END ESET

DEFINE ESET
 dgMIXT=(
  cName       = CH16                    : 'mixture name',
  rA(1:10)    = REAL [0.,300.0]          : 'array of atomic weights',
  rZ(1:10)    = REAL [0.,300.0]          : 'array of atomic numbers',
  rDens       = REAL [0.00,30.0]           : 'total density',
  nLMat       = INTE [-10,10]           : 'signed number of constituents',
  rWMat(1:10) = REAL [0.00,100.]          : 'array of weights or mulecules')

 : 'mixtures and compounds';
END ESET
DEFINE ESET
 dgTMED=(
  cName       = CH16                     : 'tracking medium name',
  iSVol       = INTE  [00,1]             : 'sensitivity flag',
  iField      = INTE  [00,3]             : 'magn. field flag',
  rFieldMax   = REAL [0.,30.0]           : 'max. field in kGauss',
  rTMaxFD     = REAL [0.,30.0]           : 'max. tracking step angle',
  rStepMax    = REAL [0.,100.0]          : 'max. step size',
  rDEEmax     = REAL [0.,1.00]           : 'max. fract. Energ. loss per step',
  rEpsil      = REAL [0.,1.00]           : 'boundary crossing precision',
  rStMin      = REAL [0.,1.00]           : 'min step size')
 : 'Tracking Medium' ;
END ESET

DEFINE ESET
 dgTRACK=(
  iField      = INTE  [00,3]             : 'explicit magn. field flag',
  rFieldMax   = REAL [0.,30.0]           : 'max. field in kGauss',
  rTMaxFD     = REAL [0.,30.0]           : 'max. tracking step angle',
  rStepMax    = REAL [0.,100.0]          : 'max. step size',
  rDEEmax     = REAL [0.,1.00]           : 'max. fract. Energ. loss per step',
  rEpsil      = REAL [0.,1.00]           : 'boundary crossing precision',
  rStMin      = REAL [0.,1.00]           : 'min step size')

 : 'tracking medium parameters';
END ESET

DEFINE ESET
 dgTPAR=(
  rCutGam     = REAL [0.,99.0000]     : 'Gamma cut',
  rCutEle     = REAL [0.,99.0000]     : 'Electron cut',
  rCutNeu     = REAL [0.,99.0000]     : 'Neutron cut',
  rCutHad     = REAL [0.,99.0000]     : 'Hadron cut',
  rCutMuo     = REAL [0.,99.0000]     : 'Muon cut',
  rBCutE      = REAL [0.,99.0000]     : 'electron bremssstr. cut',
  rBCutM      = REAL [0.,99.0000]     : 'muon, hadron bremssstr. cut',
  rDCutE      = REAL [0.,99.0000]     : 'delta ray by electrons cut',
  rDCutM      = REAL [0.,99.0000]     : 'delta ray by muons cut',
  rPPCutM     = REAL [0.,99.0000]     : 'pair prod. by muons cut',
  iPair       = INTE [0,2]            : 'pair production flag',
  iComp       = INTE [0,2]            : 'Compton sscattering flag',
  iPhot       = INTE [0,2]            : 'photo effect flag',
  iPFis       = INTE [0,2]            : 'photofission flag',
  iDRay       = INTE [0,2]            : 'Delta ray flag',
  iAnni       = INTE [0,2]            : 'Annihilation flag',
  iBrem       = INTE [0,2]            : 'Bremsstrahlung flag',
  iHadr       = INTE [0,2]            : 'Hadronic process flag',
  iMuNu       = INTE [0,2]            : 'Muon Nucl. Int. flag',
  iLoss       = INTE [0,2]            : 'Energy loss flag',
  iDcay       = INTE [0,2]            : 'decay flag',
  iMuls       = INTE [0,2]            : 'Multiple sscattering flag')
 : 'special tracking parameters';
END ESET

DEFINE ESET
 dgDigiTPAR=(
  DigiName	= CHA4 'VC'|'FC'|'Prop'|'BC'|'Cer'|'Hodo'|'TRD'|'Calo'
		: 'digitization option controlling sp. param. selection')
 : 'table to switch sp. params based on digitization options';
END ESET

DEFINE ESET
 dgVOLU=(
  cName     = CHA4                    : 'Unique 4-character volume name',
  cShape    = CHA4  'BOX'|'TRD1'|'TRD2'|'TRAP'|'TUBE'|'TUBS'|'CONS'|'CONE'|
'SPHE'|'PARA'|'PGON'|'PCON'|'GTRA'    : '4-character name of system shape',
  rPar(1:12)= REAL [-2000.0,2000.]    : 'Array of shape parameters',
  nPar      = INTE [1,12]             : 'Number of shape parameters')

 : 'create volume';
END ESET


DEFINE ESET
 dgPOS=(
  nNR         = INTE [1,500]            : 'user copy number of volume',
  rX          = REAL [-2000.0,2000.]    : 'x position of volume in mother',
  rY          = REAL [-2000.0,2000.]    : 'y position of volume in mother',
  rZ          = REAL [-2000.0,2000.]    : 'z position of volume in mother',
  cOnly       = CHA4   'ONLY'|'MANY'    : 'overlapping volume flag')

 : 'Position of volume inside mother';
END ESET

DEFINE ESET
 dgDVN=(
  cName     = CHA4                    : 'Unique 4-character volume name',
  nDiv      = INTE [1,500]            : 'Number of divisions',
  iAxis     = INTE [1,3]              : 'axis number for slicing')

 : 'Division of a volume into cells';
END ESET

DEFINE ESET
 dgDVN2=(
  cName     = CHA4                    : 'Unique 4-character volume name',
  nDiv      = INTE [1,500]            : 'Number of divisions',
  iAxis     = INTE [1,3]              : 'axis number for slicing',
  rOffset   = REAL                    : 'Offset for divisions which dont fit',
  iTmed     = INTE                    : 'Tracking Medium')

 :  'For dividing Wire Planes';
END ESET

DEFINE ESET
 dgROTM=(
  aTheta1    = REAL [0.0,180.]       : 'polar angle for axis I',
  aPhi1      = REAL [0.0,360.]       : 'azimuthal angle for axis I',
  aTheta2    = REAL [0.0,180.]       : 'polar angle for axis II',
  aPhi2      = REAL [0.0,360.]       : 'azimuthal angle for axis II',
  aTheta3    = REAL [0.0,180.]       : 'polar angle for axis III',
  aPhi3      = REAL [0.0,360.]       : 'azimuthal angle for axis III')
 : 'Rotation Matrix';
END ESET

DEFINE ESET
 dgTransMat=(
  r11           = REAL                  : '3x3 transformation matrix for each dg
ROTM',
  r12           = REAL,
  r13           = REAL,
  r21           = REAL,
  r22           = REAL,
  r23           = REAL,
  r31           = REAL,
  r32           = REAL,
  r33           = REAL)
  : '3x3 matrix to convert (rotated) detector coordinates to global H...';
END ESET

DEFINE ESET
 dgDETS=(
  iType     = INTE [1,1000000]    :
  'detector type (drift;>50=prop;>100=calor.; i/100=columns i/10000=rows)',
  aWCos     = REAL [-1.00,1.]     :
             'abs.-angle cosinus of coordinate (resp. of divided module)',
  aWSin     = REAL [-1.000,1.]    :
             'abs.-angle sinus of coordinate (resp. of divided module)',
  aRot      = REAL :
             ' y rotation of the plane (for the VCs)',
  nWires    = INTE [10,2000]      : 'number of wires',
  pW1       = REAL [-200.0,-0.1]
                        : 'abs. pos. of first wire along plane coord.',
  sWire     = REAL [0.005,20.]    : 'sense wire spacing',
  aResolu   = REAL [-1.,.1]       : 'spatial resolution (-1 for no resol.)')
    : 'detector definition for digitalization (DETS for HITS and DIGITS)';
END ESET

/* this is crap:
DEFINE ESET
 dgSETS=(
  cName     = CHA4                    : 'Unique 4-character set name')
 : 'Detector SETS (HITS and DIGITS)';
END ESET
*/

DEFINE ESET
 dgDETINFO=(
  cName     = CHA4                 : 'Unique 4-character volume name',
  cType     = CHA4 'u'|'v'|'x'|'y' : 'type of plane',
  sXSiz     = REAL [-400.,400.]    : 'hor. size of plane (block)',
  sYSiz     = REAL [-400.,400.]    : 'vert. size of plane (block)',
  pXC1      = REAL [-400.0,10.]    :
                             'x-position of center of plane (first block)',
  pYC1      = REAL [-400.0,10.]    :
                             'y-position of center of plane (first block)',
  pZ        = REAL [-300.0,500.]   :
                             'z-position of plane (block center)')
    : 'definition for drawing digitized hits of detector';
END ESET


DEFINE RSET
 (dgVOLU  [1,1] -> [0,*] dgTMED)   : 'Volumes consist of tracking medium';
 (dgPOS   [1,1] -> [0,*] dgVOLU)   : 'volume shapes are positioned';
 (dgPOS   [1,1] -> [0,*] dgVOLU |
                -> [0,*] dgDVN
                  BY Mother)	   : 'volumes are pos. inside mothers';
 (dgPOS   [0,1] -> [0,*] dgROTM)   : 'volumes are rotated';
 (dgDVN   [1,1] -> [0,*] dgVOLU |
                -> [0,*] dgDVN
                  BY Mother)       : 'mothers may be divided';
 (dgDVN2  [1,1] -> [0,*] dgVOLU)   : 'general division of a volume';
 (dgTMED  [1,1] -> [0,*] dgMATE |
                -> [0,*] dgMIXT
                  BY Material)     : 'tracking media consist of material';
 (dgTMED [0,1] -> [1,*] dgTRACK)   : 'All track. med. have these parameters';
 (dgTMED [0,1] -> [0,*] dgDigiTPAR): 'some track. med. have special parameters';
 (dgDigiTPAR[0,1] -> [0,*] dgTPAR BY NoneTPAR) : 'TPAR for NONE digitization';
 (dgDigiTPAR[0,1] -> [0,*] dgTPAR BY FastTPAR) : 'TPAR for FAST digitization';
 (dgDigiTPAR[0,1] -> [0,*] dgTPAR BY ExptTPAR) : 'TPAR for EXPT digitization';
 (dgDigiTPAR[0,1] -> [0,*] dgTPAR BY RealTPAR) : 'TPAR for REAL digitization';
 (dgDETS [1,1] -> [1,1] dgDETINFO) : 'each detector has more information';
 (dgDETINFO [1,1] -> [0,1] dgVOLU |
                  -> [0,1] dgDVN  |
                  -> [0,1] dgDVN2
                  BY Volume)    : 'detectors have a certain kind of volume';
 (dgDETS [1,1] -> [1,*] dgTransMat) : 'Detector can be rotated';
END RSET

DEFINE DATAFLOW
   dgMaterials  = {dgMATE},{dgMIXT},{dgTMED},{dgTRACK},{dgTPAR},{dgDigiTPAR}
           : 'Geant material / Tracking tables';
   dgVolumes    = {dgVOLU},{dgDVN},{dgPOS},{dgROTM},{dgDVN2}
        ,{dgTransMat}
           : 'Geant volume tables';
   dgGeometries  = dgMaterials,dgVolumes,{dgDETS},{dgDETINFO}
        : 'Geant Geometry tables';
END DATAFLOW

END SUBSCHEMA
