
!=====================================================================
! SECTION ONE: First write the user routines needed by the HANNA frame
!=====================================================================

	integer function user_args ()

	implicit none
#include "frame.inc"

! Initialize return code to success

	user_args = 0

! Create user switches

! ... filename base for ntuple files
	call addarg('-out','char*80',
     +		'filename base for output files [D: nt]','nt')

! ... control of ntuple splitting
	call addarg('-split','',
     +		'split ntuple file every nsplit events','')
	call addarg('-nsplit','integer',
     +	  'no. of records in each split ntuple [D: 20000]','20000')
	call addarg('-runsplit','',
     +	  'split ntuple file every run (supercedes -split)','')

! ... good_event and good_track criteria
	call addarg('-scat','',
     +		'only store events with scattered lepton in acceptance','')
	call addarg('-accep','',
     +		'only store tracks in acceptance','')
	call addarg('-lund','',
     +		'only store stable (cType = lund) tracks','')
	call addarg('-charge','',
     +		'only store charged tracks','')

! ... acceptance definition
	call addarg('-pmin','real',
     +		'acceptance: min momentum [D: 0]','0')
	call addarg('-thmin','real',
     +		'acceptance: min theta [D: 0]','0')
	call addarg('-thmax','real',
     +		'acceptance: max theta [D: pi]','3.14159')
	call addarg('-thxmax','real',
     +		'acceptance: max |theta_x| [D: pi/2]','1.5708')
	call addarg('-thymin','real',
     +		'acceptance: min |theta_y| [D: 0]','0')
	call addarg('-thymax','real',
     +		'acceptance: max |theta_y| [D: pi/2]','1.5708')

	return
	end

!=====================================================================

	integer function user_init ()

	implicit none
#include "frame.inc"

	integer iok
	integer lnblnk

! Declare routines for retrieval of command-line arguments

	integer		getarg_bool, getarg_int
	real		getarg_real
	character*80	getarg_char

! Initialize return code to success

	user_init = 0

! Retrieve any command line arguments that were declared

	file_base = 'nt'
	file_base = getarg_char('-out')
	nt_runsplit = ( getarg_bool('-runsplit') .gt. 0 )
	if (nt_runsplit) then
	  nt_split = .false.
	else
	  nt_split = ( getarg_bool('-split') .gt. 0 )
	  nt_size = -1
	  if ( nt_split ) nt_size = getarg_int('-nsplit') 
	endif
	req_scat = ( getarg_bool('-scat') .gt. 0 )
	req_accep = ( getarg_bool('-accep') .gt. 0 )
	req_lund = ( getarg_bool('-lund') .gt. 0 )
	req_charge = ( getarg_bool('-charge') .gt. 0 )
	accep_pmin = getarg_real('-pmin')
	accep_thmin = getarg_real('-thmin')
	accep_thmax = getarg_real('-thmax')
	accep_thxmax = getarg_real('-thxmax')
	accep_thymin = getarg_real('-thymin')
	accep_thymax = getarg_real('-thymax')

! Clear job-level counters 

	c_nev = 0
	c_nevgen = 0
	c_ngoodtrack = 0
	ifile = 0
	nt_num = 0
	nt_icycle = 999999

! Open and book new ntuple (unless we are splitting at runs)

	if (.not.nt_runsplit) call book_ntuple

	return
	end

!=====================================================================

	integer function user_runinit ( runnum )

	implicit none
#include "frame.inc"

	integer runnum, i

! Initialize return code to success

	user_runinit = 0

! Reset run-level counters

! NOTE: runnum is always 0 when reading Monte Carlo GAF files
	run = runnum
	cr_nev = 0
	cr_nevgen = 0
	cr_ngoodtrack = 0

! Speak to the human

	ifile = ifile + 1
!	write(6,'(/a,i3,a)') 'Processing input file #', ifile, ' ...'
	write(6,'(/a,i3,a,i5)') 'Processing input file #', ifile, 
     +		' ... run number = ', runnum

! If we're splitting the ntuple every run, start a new ntuple file

	if (nt_runsplit) call book_ntuple

	return
	end

!=====================================================================

	integer function user_event ( runnum, eventnum, 
     +		eventtime, eventtimemu, index, cur1, cur2 )

	implicit none
#include "frame.inc"

	integer runnum, eventnum, eventtime, eventtimemu
	integer	index, cur1, cur2
	logical good_event

! Initialize return code to success

	user_event = 0

! Increment job-level and run-level event counters

! ... number of events processed
	c_nev = c_nev + 1
	cr_nev = cr_nev + 1

! ... number of events generated internally by GMC
	call fettab(mcEvent,ID,1)
	c_nevgen = c_nevgen + mcEvent_iEvGen
	cr_nevgen = cr_nevgen + mcEvent_iEvGen

! Fill the ntuple will every track of interest

	call fill_ntuple

! Keep the human informed

	if (mod(cr_nev,500).eq.0) 
     +		print *, 'processed ', cr_nev, ' events'

	return
	end

!=====================================================================

	integer function user_runend ( retcode )

	implicit none
#include "frame.inc"

	integer i, lun, retcode

! Initialize return code to whatever was handed to us

	user_runend = retcode

! Produce run-level counter report

	lun = 6
	write(lun,900) 
!	write(lun,901) 'RUN: ', run
	write(lun,901) 'FILE: ', ifile
	write(lun,901) 'Number of events processed = ', cr_nev
	write(lun,901) 'Number of generated events = ', cr_nevgen
	write(lun,901) 'Number of good tracks      = ', cr_ngoodtrack
	write(lun,900) 

900	format (1x,60('-'))
901	format (1x,a,i6)

! No problems

	user_runend = 0
	retcode = 0

	return
	end

!=====================================================================

	integer function user_end ( retcode )

	implicit none
#include "frame.inc"

	integer i, j, n, lun, retcode
	integer lnblnk

! Initialize return code to whatever was handed to us

	user_end = retcode

! Job-level counter report

	lun = 6
	write(lun,900) 
	write(lun,901) 'SUMMARY: ALL RUNS PROCESSED'
	write(lun,901) 'Number of events processed = ', c_nev
	write(lun,901) 'Number of generated events = ', c_nevgen
	write(lun,901) 'Number of good tracks      = ', c_ngoodtrack
	write(lun,900) 

900	format (/1x,60('='))
901	format (1x,a,i6)

! Save ntuple and close file

	call hrout (0, nt_icycle,' ')
	call hrend (CTOP)
	close (NT_LUN)
	write(6,'(2a)') '  >>> closed ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))

! No problems

	user_end = 0
	retcode = 0

	return
	end

!=====================================================================
! We don't use these user functions, just need empty templates

	integer function user_burstinit ( runnum, burstnum, 
     +		bursttime, bursttimemu, burstgood )

	implicit none
	integer runnum, burstnum, bursttime, bursttimemu, burstgood

! Initialize return code to success

	user_burstinit = 0

	return
	end

!---------------------------------------------------------------------

	integer function user_burstend ( retcode )

	implicit none
	integer retcode

! Initialize return code to success

	user_burstend = retcode

	return
	end
	
!---------------------------------------------------------------------

	integer function user_slow ( name, starttime, starttimemu,
     +		endtime, endtimemu, flag_first, status )

	implicit none
	character*(*)	name
	integer		starttime, starttimemu, endtime, endtimemu
	integer		flag_first, status

! Initialize return code to success

	user_slow = 0

	return
	end


!=====================================================================
! SECTION TWO: Following are codes specific to this program, 
!		e.g. to book and fill the ntuple, and perform calculations
!=====================================================================

!=====================================================================
! Open a new ntuple file and book the ntuple

	subroutine book_ntuple

	implicit none
#include "frame.inc"

	integer i, iok
	integer lnblnk
	character*80 string

! Update the ntuple file counter

	nt_num = nt_num + 1

! Construct the output filename from the user-specified filename base

	i = lnblnk(file_base)

! ... if we're splitting at every run, use base+runnumber
	if (nt_runsplit) then
	  write(nt_file,'(a,i5.5,a)') file_base(1:i), run, '.rz'

! ... if we're splitting every n records, update the ntuple counter 
! ... and use it in the filename
	else if (nt_split) then
	  write(string,'(i6)') nt_num
	  call cleft(string,1,80)
	  write(nt_file,99) file_base(1:i), string(1:lnblnk(string))
99	  format(a,'_',a,'.rz')

! ... if we are not doing any splitting of the ntuple, use the base directly
	else 
	  write(nt_file,'(2a)') file_base(1:i), '.rz'

	endif

! Open ntuple file

! ... if this is not the first file, we have to save and close the previous one
	if (nt_num .gt. 1) then
	  call hrout (0, nt_icycle,' ')
	  call hrend (CTOP)
	  close (NT_LUN)
	  write(6,'(2a)') '  >>> closed ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))
	endif

! ... open the new file
        call hropen (NT_LUN,CTOP,nt_file,'N',LRECL,iok)
        if (iok .ne. 0) then
	  print *, 'ERROR: Could not open output rz file.'
	  stop
	endif
	write(6,'(/2a)') '  >>> opened new ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))

! Book ntuple

	call hbookn(NT1,NT1_TITLE,NVAR,CTOP,5000,chtags)

	return
	end

!=====================================================================
! Fill ntuple records with information from current event

	subroutine fill_ntuple

	implicit none
#include "frame.inc"

	integer it, indctype, icur1, icur2
	real	qx, qy, qz, qmag, ppar
	logical	check_accep

! Fill event-level ntuple variables

! ... retrieve the table information
	mcEvent_ID = 1
	call gettab (mcEvent)
	mcBeam_ID = 1
	call gettab (mcBeam)

! ... shuffle all interesting values to ntuple variables
	ievent = mcEvent_iEvent
	Q2 = mcEvent_Q2
	W2 = mcEvent_w2
	nu = mcEvent_nu
	x = mcEvent_x
	y = mcEvent_y
	weight = mcEvent_weight
	quark = mcEvent_qflavour
	polB = mcEvent_PolB
	polT = mcEvent_PolT

! Locate the scattered-lepton track and fill some more variables

! ... locate the track using an ADAMO index
	indctype = getind(mcTrack,'cType')
	mcTrack_cType = 'scat'
	call seltab(mcTrack,indctype,icur1,icur2)
	if (icur1.ne.icur2) stop 'ERROR: could not find scat beam track'

! ... check acceptance of lepton, and leave now if req_scat is on
	if (check_accep) then
	  scat_accep = 1.
	else
	  scat_accep = 0.
	  if (req_scat) return
	endif

! ... fill more event-level ntuple variables 
	scat_p = mcTrack_P
	scat_theta = mcTrack_The
	scat_phi = mcTrack_Phi

! ... determine the q-vector, will need it for pT calculations below
	qx = -mcTrack_Px
	qy = -mcTrack_Py
	qz = mcBeam_EBe - mcTrack_Pz
	qmag = sqrt(qx**2+qy**2+qz**2)

! Loop over tracks, storing an ntuple record for each interesting one

	itgood = 0
	do it = 1, COUTAB(mcTrack)

! ... retrieve track data from mcTrack table
	  call fettab(mcTrack,ID,it)

! ... do we want to store this track? if not, move on the the next one
	  if (req_lund) then
	    if (mcTrack_cType.ne.'lund') goto 99
	  endif
	  if (req_charge) then
	    if (mcTrack_iCharge.eq.0) goto 99
	  endif
	  if (check_accep) then
	    accep = 1.
	  else
	    accep = 0.
	    if (req_accep) goto 99
	  endif
	  itgood = itgood + 1
	  
! ... store info in local variables
	  iltype = mcTrack_iLType
	  p = mcTrack_P
	  theta = mcTrack_The
	  phi = mcTrack_Phi
	  z = mcTrack_E / nu
	  ppar = (mcTrack_Px*qx + mcTrack_Py*qy + mcTrack_Pz*qz) / qmag
	  pT = mcTrack_P**2 - ppar**2
	  if (pT.gt.0) then
	    pT = sqrt(pT)
	  else
	    pT = 0.
	  endif

! ... retrieve this track's parent to determine its Lund type
	  mcTrack_ID = mcTrack_Parent
	  call gettab(mcTrack)
	  ilparent = mcTrack_iLType

! ... this is a good track, update counters
	  c_ngoodtrack = c_ngoodtrack + 1
	  cr_ngoodtrack = cr_ngoodtrack + 1
	  
! ... before storing, check if it's time to start a new ntuple file
	  if (nt_split .and. (c_ngoodtrack.gt.1)) then
	    if (mod(c_ngoodtrack,nt_size).eq.1) call book_ntuple
	  endif

! ... store the track as a new record
	  call hfn (NT1,values)

! ... finished loop over tracks
99	  continue
	  
	enddo

	return
	end

!=====================================================================
! Check whether track presently in the mcTrack WCB is within acceptance

	logical function check_accep()

	implicit none
#include "frame.inc"

	real	thx, thy

	check_accep = .false.
	if (mcTrack_P.lt.accep_pmin) return
	if (mcTrack_The.lt.accep_thmin) return
	if (mcTrack_The.gt.accep_thmax) return
	thx = 0.
	if (mcTrack_Px.ne.0..and.mcTrack_Pz.ne.0.) 
     +		thx = atan2(mcTrack_Px,mcTrack_Pz)
	if (abs(thx).gt.accep_thxmax) return
	thy = 0.
	if (mcTrack_Py.ne.0..and.mcTrack_Pz.ne.0.) 
     +		thy = atan2(mcTrack_Py,mcTrack_Pz)
	if (abs(thy).gt.accep_thymax) return
	if (abs(thy).lt.accep_thymin) return
	check_accep = .true.

	return
	end


